-- --------------------
-- TellMeWhen
-- Originally by Nephthys of Hyjal <lieandswell@yahoo.com>
-- Other contributions by
-- Oozebull of Twisting Nether
-- Banjankri of Blackrock
-- Predeter of Proudmoore
-- Xenyr of Aszune
-- Cybeloras of Mal'Ganis
-- --------------------


-- -------------
-- ADDON GLOBALS
-- -------------

local LBF = LibStub("LibButtonFacade", true)
if LBF then
	LBF:RegisterSkinCallback("TellMeWhen", TellMeWhen_SkinCallback, self);
end

TellMeWhen = {};

TELLMEWHEN_VERSION = "1.4.7";
TELLMEWHEN_MAXGROUPS = 8;
TELLMEWHEN_MAXROWS = 7;
TELLMEWHEN_ICONSPACING = 0;
TELLMEWHEN_UPDATE_INTERVAL = 0.05;

local GetSpellCooldown = GetSpellCooldown;
local GetSpellInfo = GetSpellInfo;
local IsUsableSpell = IsUsableSpell;
local UnitPower = UnitPower;
local SetMinMaxValues = SetMinMaxValues;
local SetValue = SetValue;
local SetTexCoord = SetTexCoord;
local SetStatusBarColor = SetStatusBarColor;
local IsSpellInRange = IsSpellInRange;
local UnitIsEnemy = UnitIsEnemy;
local UnitAura = UnitAura;
local UnitReaction = UnitReaction;
local UnitIsEnemy = UnitIsEnemy;
local _,pclass = UnitClass("Player")

TellMeWhen_Icon_Defaults = {
	BuffOrDebuff		= "HELPFUL",
	BuffShowWhen		= "present",
	CooldownShowWhen	= "usable",
	CooldownType		= "spell",
	Enabled				= false,
	Name				= "",
	OnlyMine			= false,
	ShowTimer			= false,
	ShowPBar			= false,
	ShowCBar			= false,
	InvertBars			= false,
	--DurationAndCD		= false,
	Type				= "",
	Unit				= "player",
	WpnEnchantType		= "mainhand",
	UnitReact			= 0,
	Conditions			= {},
	Alpha				= 100,
	RangeCheck			= false,
	ManaCheck			= false,
};


TellMeWhen_Group_Defaults = {
	Enabled			= false,
	Scale			= 2.0,
	Rows			= 1,
	Columns			= 4,
	Icons			= {},
	OnlyInCombat	= false,
	PrimarySpec		= true,
	SecondarySpec	= true,
	LBFGroup		= false,
	Stance			= {},
	Point			= {},
};

for iconID = 1, TELLMEWHEN_MAXROWS*TELLMEWHEN_MAXROWS do
	TellMeWhen_Group_Defaults["Icons"][iconID] = TellMeWhen_Icon_Defaults;
end;

TellMeWhen_Defaults = {
	Version 		= 	TELLMEWHEN_VERSION,
	Locked 			= 	false,
	Groups 			= 	{},
	Interval		=	TELLMEWHEN_UPDATE_INTERVAL,
	CDCOColor 		= 	{r=0,g=1,b=0,a=1},
	CDSTColor 		= 	{r=1,g=0,b=0,a=1},
	Texture			=	"Interface\\TargetingFrame\\UI-StatusBar",
	TextureName 	= 	"Blizzard",
	DrawEdge		=	false,
};

for groupID = 1, TELLMEWHEN_MAXGROUPS do
	TellMeWhen_Defaults["Groups"][groupID] = TellMeWhen_Group_Defaults;
	if (groupID == 1) then
		TellMeWhen_Defaults["Groups"][groupID].Enabled = true;
	end
end

function TellMeWhen_Test(stuff)
	if ( stuff ) then
		DEFAULT_CHAT_FRAME:AddMessage("TellMeWhen test: "..stuff);
	else
		DEFAULT_CHAT_FRAME:AddMessage("TellMeWhen test: "..self:GetName());
	end
end

TellMeWhen_BuffEquivalencies = {};

TellMeWhen_BuffEquivalencies["Bleeding"] = "Pounce Bleed;Rake;Rip;Lacerate;Rupture;Garrote;Savage Rend;Rend;Deep Wound";
TellMeWhen_BuffEquivalencies["VulnerableToBleed"] = "Mangle (Cat);Mangle (Bear);Trauma";
TellMeWhen_BuffEquivalencies["Incapacitated"] ="Gouge;Maim;Repentance;Reckless Charge;Hungering Cold";
TellMeWhen_BuffEquivalencies["StunnedOrIncapacitated"] ="Gouge;Maim;Repentance;Reckless Charge;Hungering Cold;Bash;Pounce;Starfire Stun;Intimidation;Impact;Hammer of Justice;Stun;Blackout;Kidney Shot;Cheap Shot;Shadowfury;Intercept;Charge Stun;Concussion Blow;War Stomp";
TellMeWhen_BuffEquivalencies["Stunned"] ="Reckless Charge;Bash;Pounce;Starfire Stun;Intimidation;Impact;Hammer of Justice;Stun;Blackout;Kidney Shot;Cheap Shot;Shadowfury;Intercept;Charge Stun;Concussion Blow;War Stomp";
TellMeWhen_BuffEquivalencies["DontMelee"] ="Berserk;Evasion;Shield Wall;Retaliation;Dispersion;Hand of Sacrifice;Hand of Protection;Divine Shield;Divine Protection;Ice Block;Icebound Fortitude;Cyclone;Banish";
TellMeWhen_BuffEquivalencies["ImmuneToStun"] ="Divine Shield;Ice Block;The Beast Within;Beastial Wrath;Icebound Fortitude;Hand of Protection;Cyclone;Banish";
TellMeWhen_BuffEquivalencies["ImmuneToMagicCC"] ="Divine Shield;Ice Block;The Beast Within;Beastial Wrath;Cyclone;Banish";
TellMeWhen_BuffEquivalencies["FaerieFires"] ="Faerie Fire;Faerie Fire (Feral)";
TellMeWhen_BuffEquivalencies["MovementSlowed"] = "Incapacitating Shout;Chains of Ice;Icy Clutch;Slow;Daze;Hamstring;Piercing Howl;Wing Clip;Ice Trap;Frostbolt;Cone of Cold;Blast Wave;Mind Flay;Crippling Poison;Deadly Throw;Frost Shock;Earthbind;Curse of Exhaustion";
TellMeWhen_BuffEquivalencies["MeleeSlowed"] = "Rocket Burst;Infected Wounds;Judgements of the Just;Earth Shock;Thunder Clap;Icy Touch";
--Much of these are thanks to Malazee @ Petopia' chart: http://img204.imageshack.us/img204/4902/cataraidbuffs.jpg
--[[
TellMeWhen_BuffEquivalencies["IncreasedStats"]
TellMeWhen_BuffEquivalencies["IncreasedDamage"]
TellMeWhen_BuffEquivalencies["IncreasedCrit"]
TellMeWhen_BuffEquivalencies["IncreasedAttackPower"]
TellMeWhen_BuffEquivalencies["IncreasedSpellPowersix"]
TellMeWhen_BuffEquivalencies["IncreasedSpellPowerten"]
TellMeWhen_BuffEquivalencies["IncreasedSpellPowerboth"]
TellMeWhen_BuffEquivalencies["IncreasedPhysicalHaste"]
TellMeWhen_BuffEquivalencies["IncreasedSpellHaste"]
TellMeWhen_BuffEquivalencies["BurstHaste"]
TellMeWhen_BuffEquivalencies["BonusAgiStr"]
TellMeWhen_BuffEquivalencies["BonusStamina"]
TellMeWhen_BuffEquivalencies["BonusArmor"]
TellMeWhen_BuffEquivalencies["BonusMana"]
TellMeWhen_BuffEquivalencies["ManaRegen"]
TellMeWhen_BuffEquivalencies["BurstManaRegen"]
TellMeWhen_BuffEquivalencies["PushbackResistance"]

TellMeWhen_BuffEquivalencies["PhysicalDamageTaken"]
TellMeWhen_BuffEquivalencies["SpellDamageTaken"]
TellMeWhen_BuffEquivalencies["SpellCritTaken"]
TellMeWhen_BuffEquivalencies["BleedDamageTaken"]
TellMeWhen_BuffEquivalencies["ReducedAttackSpeed"]
TellMeWhen_BuffEquivalencies["ReducedCastingSpeed"]
TellMeWhen_BuffEquivalencies["ReducedArmor"]
TellMeWhen_BuffEquivalencies["ReducedHealing"]
TellMeWhen_BuffEquivalencies["ReducedPhysicalDamageDone"]
--TellMeWhen_BuffEquivalencies["CastingSlowed"] = "";]]


-- ---------------
-- EXECUTIVE FRAME
-- ---------------

function TellMeWhen_OnEvent(self, event,...)
	if ( event == 'VARIABLES_LOADED' ) then
		SlashCmdList["TELLMEWHEN"] = TellMeWhen_SlashCommand;
		SLASH_TELLMEWHEN1 = "/tellmewhen";
		SLASH_TELLMEWHEN2 = "/tmw";
		if ( not TellMeWhen_Settings ) then
			TellMeWhen_Settings = CopyTable(TellMeWhen_Defaults);
			TellMeWhen_Settings["Groups"][1]["Enabled"] = true;
		elseif ( TellMeWhen_Settings["Version"] < TELLMEWHEN_VERSION ) then
			TellMeWhen_SafeUpgrade();
		end
	elseif ( event == "PLAYER_LOGIN" ) or ( event == "PLAYER_ENTERING_WORLD" ) then
		self:RegisterEvent("PLAYER_TALENT_UPDATE");
		self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
		TellMeWhen_Update();
	elseif ( event == "PLAYER_TALENT_UPDATE")  then
		TellMeWhen_Update();
	elseif ( event == "PET_BAR_UPDATE" ) then
		TellMeWhen_PetEvent()
	end
end

function TellMeWhen_SafeUpgrade()
	if (TellMeWhen_Settings["Version"] < "1.1.4") then
		TellMeWhen_Settings = CopyTable(TellMeWhen_Defaults);
		TellMeWhen_Settings["Groups"][1]["Enabled"] = true;
		TellMeWhen_Settings["Version"] = TELLMEWHEN_VERSION;
	elseif (TellMeWhen_Settings["Version"] < "1.2.0") then
	TellMeWhen_Settings = TellMeWhen_AddNewSettings(TellMeWhen_Settings, TellMeWhen_Defaults);
		for groupID = 1, TELLMEWHEN_MAXGROUPS do
			if (groupID < 5) then
				oldgroupSettings = TellMeWhen_Settings["Spec"][1]["Groups"][groupID];
				TellMeWhen_Settings["Groups"][groupID]["SecondarySpec"] = false;
			else
				local temp_groupID = groupID-4;
				TellMeWhen_Settings["Groups"][groupID]["PrimarySpec"] = false;
				oldgroupSettings = TellMeWhen_Settings["Spec"][2]["Groups"][temp_groupID];
			end
			if (oldgroupSettings) then
				TellMeWhen_Settings["Groups"][groupID]["Enabled"] = oldgroupSettings.Enabled;
				TellMeWhen_Settings["Groups"][groupID]["Scale"] = oldgroupSettings.Scale;
				TellMeWhen_Settings["Groups"][groupID]["Rows"] = oldgroupSettings.Rows;
				TellMeWhen_Settings["Groups"][groupID]["Columns"] = oldgroupSettings.Columns;
				TellMeWhen_Settings["Groups"][groupID]["OnlyInCombat"] = oldgroupSettings.OnlyInCombat;
			end

			for iconID = 1, TELLMEWHEN_MAXROWS*TELLMEWHEN_MAXROWS do
				if (oldgroupSettings) then
					oldiconSettings = oldgroupSettings["Icons"][iconID];
					if (oldiconSettings) then
						iconSettings = TellMeWhen_Settings["Groups"][groupID]["Icons"][iconID];
						iconSettings.BuffOrDebuff = oldiconSettings.BuffOrDebuff;
						iconSettings.BuffShowWhen = oldiconSettings.BuffShowWhen;
						iconSettings.CooldownShowWhen = oldiconSettings.CooldownShowWhen;
						iconSettings.CooldownType = oldiconSettings.CooldownType;
						iconSettings.Enabled = oldiconSettings.Enabled;
						iconSettings.Name = oldiconSettings.Name;
						iconSettings.OnlyMine = oldiconSettings.OnlyMine;
						iconSettings.ShowTimer = oldiconSettings.ShowTimer;
						iconSettings.Type = oldiconSettings.Type;
						iconSettings.Unit = oldiconSettings.Unit;
						iconSettings.WpnEnchantType = oldiconSettings.WpnEnchantType;
					end
				end
				if (iconSettings.Name == "" and iconSettings.type ~= "wpnenchant") then
					TellMeWhen_Settings["Groups"][groupID]["Icons"][iconID]["Enabled"] = false;
				end
			end
		end
	TellMeWhen_Settings["Spec"] = nil;  -- Remove "Spec" {}
	end
	-- End convert 1.1.6 to 1.2.0
	if (TellMeWhen_Settings["Version"] < "1.2.5") then
		--no settings added yet in 1.2.5
		local iconSettings
		for groupID = 1, TELLMEWHEN_MAXGROUPS do
			TellMeWhen_Settings["Groups"][groupID]["LBFGroup"] = false;

			for iconID = 1, TELLMEWHEN_MAXROWS*TELLMEWHEN_MAXROWS do
				 -- TellMeWhen_Settings["Groups"][groupID]["Icons"][iconID]

			end
		end
	end
	
	if (TellMeWhen_Settings["Version"] < "1.3.0") then
		TellMeWhen_Settings["Texture"] = "Interface\\TargetingFrame\\UI-StatusBar";
		TellMeWhen_Settings["TextureName"] = "Blizzard";
		for groupID = 1, TELLMEWHEN_MAXGROUPS do
			for iconID = 1, TELLMEWHEN_MAXROWS*TELLMEWHEN_MAXROWS do
				TellMeWhen_Settings["Groups"][groupID]["Icons"][iconID]["ShowPBar"] = false;
				TellMeWhen_Settings["Groups"][groupID]["Icons"][iconID]["ShowCBar"] = false;
				TellMeWhen_Settings["Groups"][groupID]["Icons"][iconID]["InvertBars"] = false;
			end
		end
	end
	--[[if (TellMeWhen_Settings["Version"] < "1.3.2") then
		for groupID = 1, TELLMEWHEN_MAXGROUPS do
			for iconID = 1, TELLMEWHEN_MAXROWS*TELLMEWHEN_MAXROWS do
				TellMeWhen_Settings["Groups"][groupID]["Icons"][iconID]["DurationAndCD"] = false;
			end
		end
	end]]
	if (TellMeWhen_Settings["Version"] < "1.3.3") then
		for groupID = 1, TELLMEWHEN_MAXGROUPS do
			for iconID = 1, TELLMEWHEN_MAXROWS*TELLMEWHEN_MAXROWS do
				TellMeWhen_Settings["Groups"][groupID]["Icons"][iconID]["UnitReact"] = 0;
			end
		end
	end
	if (TellMeWhen_Settings["Version"] < "1.4.0") then
		--  Convert 1.2.5 to 1.2 .5.1 settings
		for groupID = 1, TELLMEWHEN_MAXGROUPS do
			for iconID = 1, TELLMEWHEN_MAXROWS*TELLMEWHEN_MAXROWS do
				iconSettings = TellMeWhen_Settings["Groups"][groupID]["Icons"][iconID];
				if (iconSettings.Conditions == nil) then
					iconSettings.Conditions = {};
				end
				if (iconSettings.Alpha == nil) then
					iconSettings.Alpha = 100;
				end
			end
		end
	end
	if (TellMeWhen_Settings["Version"] < "1.4.1") then
		TellMeWhen_Settings["Interval"] = TELLMEWHEN_UPDATE_INTERVAL;
	end
	if (TellMeWhen_Settings["Version"] < "1.4.3") then
		for groupID = 1, TELLMEWHEN_MAXGROUPS do
			TellMeWhen_Settings["Groups"][groupID]["Stance"] = {}
		end
	end
	if (TellMeWhen_Settings["Version"] < "1.4.4") then
		TellMeWhen_Settings["CDCOColor"] = {r=0,g=1,b=0,a=1}
		TellMeWhen_Settings["CDSTColor"] = {r=1,g=0,b=0,a=1}
	end
	if (TellMeWhen_Settings["Version"] < "1.4.5") then
		for groupID = 1, TELLMEWHEN_MAXGROUPS do
			TellMeWhen_Settings["Groups"][groupID]["Point"] = {}
			local group = _G["TellMeWhen_Group"..groupID];
			local p = TellMeWhen_Settings["Groups"][groupID]["Point"]
			p.point,_,p.relativePoint,p.x,p.y = group:GetPoint(1)
		end
	end
	if (TellMeWhen_Settings["Version"] < "1.4.6") then
		for groupID = 1, TELLMEWHEN_MAXGROUPS do
			for iconID = 1, TELLMEWHEN_MAXROWS*TELLMEWHEN_MAXROWS do
				TellMeWhen_Settings["Groups"][groupID]["Icons"][iconID]["RangeCheck"] = false
				TellMeWhen_Settings["Groups"][groupID]["Icons"][iconID]["ManaCheck"] = false
			end
		end
	end
	if (TellMeWhen_Settings["Version"] < "1.4.7") then
		TellMeWhen_Settings["DrawEdge"] = false
	end

	--All Upgrades Complete
	TellMeWhen_Settings["Version"] = TELLMEWHEN_VERSION;
end

function TellMeWhen_AddNewSettings(settings, defaults)
	for k, v in pairs(defaults) do
		if ( not settings[k] ) then
			if ( type(v) == "table" ) then
				settings[k] = {};
				settings[k] = TellMeWhen_AddNewSettings(settings[k], defaults[k]);
			else
				settings[k] = v;
			end
		elseif ( type(v) == "table" ) then
			settings[k] = TellMeWhen_AddNewSettings(settings[k], defaults[k]);
		end
	end
	return settings;
end

local st, co  --yep, this is kinda dumb
function TellMeWhen_Update()
	st = TellMeWhen_Settings["CDSTColor"]
	co = TellMeWhen_Settings["CDCOColor"]
	for groupID = 1, TELLMEWHEN_MAXGROUPS do
		TellMeWhen_Group_Update(groupID);
	end
end

local defaultSpells = {} -- this must be outside the do
do
	local ver = select(4, GetBuildInfo());
	if (ver >= 30000) then
		defaultSpells = {
			ROGUE=GetSpellInfo(1752), -- sinister strike
			PRIEST=GetSpellInfo(139), -- renew
			DRUID=GetSpellInfo(774), -- rejuvenation
			WARRIOR=GetSpellInfo(772), -- rend
			MAGE=GetSpellInfo(7302), -- frost armor
			WARLOCK=GetSpellInfo(687), -- demon armor
			PALADIN=GetSpellInfo(4987), -- cleanse
			SHAMAN=GetSpellInfo(324), -- lightning shield
			HUNTER=GetSpellInfo(1978), -- serpent sting
			DEATHKNIGHT=GetSpellInfo(45462) -- plague strike
		}
	else
		defaultSpells = {
			ROGUE=GetSpellInfo(1752), -- sinister strike
			PRIEST=GetSpellInfo(139), -- renew
			DRUID=GetSpellInfo(774), -- rejuvenation
			WARRIOR=GetSpellInfo(772), -- rend
			MAGE=GetSpellInfo(7302), -- frost armor
			WARLOCK=GetSpellInfo(687), -- demon armor
			PALADIN=GetSpellInfo(4987), -- cleanse
			SHAMAN=GetSpellInfo(324), -- lightning shield
			HUNTER=GetSpellInfo(1978), -- serpent sting
		}
	end
	local executiveFrame = CreateFrame("Frame", "TellMeWhen_ExecutiveFrame");
	executiveFrame:SetScript("OnEvent", TellMeWhen_OnEvent);
	executiveFrame:RegisterEvent("VARIABLES_LOADED");
	executiveFrame:RegisterEvent("PLAYER_LOGIN");
	executiveFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
	executiveFrame:RegisterEvent("PET_BAR_UPDATE");
end


-- -----------
-- GROUP FRAME
-- -----------

function TellMeWhen_Group_OnEvent(self, event,...)
	if (event == "PLAYER_REGEN_ENABLED" or event == "PLAYER_REGEN_DISABLED") then
		if self.combatcheck then
			if UnitAffectingCombat("player") then
				self:Show();
			elseif not UnitAffectingCombat("player") then
				self:Hide();
			end
		end
	elseif (event == "UPDATE_SHAPESHIFT_FORM" or event == "UPDATE_SHAPESHIFT_FORMS") then
		TellMeWhen_Stance_Check(self)
	end
end


-- Called when the configuration of the group has changed, when the addon
-- is loaded or when tmw is locked or unlocked
function TellMeWhen_Group_Update(groupID)
	
	local groupName = "TellMeWhen_Group"..groupID;
	local group = _G[groupName];
	local resizeButton = _G[groupName.."_ResizeButton"];

	local locked = TellMeWhen_Settings["Locked"];
	local genabled = TellMeWhen_Settings["Groups"][groupID]["Enabled"];
	local scale = TellMeWhen_Settings["Groups"][groupID]["Scale"];
	local rows = TellMeWhen_Settings["Groups"][groupID]["Rows"];
	local columns = TellMeWhen_Settings["Groups"][groupID]["Columns"];
	local onlyInCombat = TellMeWhen_Settings["Groups"][groupID]["OnlyInCombat"];
	
	local currentSpec = GetActiveTalentGroup();
	local activePriSpec = TellMeWhen_Settings["Groups"][groupID]["PrimarySpec"];
	local activeSecSpec = TellMeWhen_Settings["Groups"][groupID]["SecondarySpec"];
	group.correctspec = true
	if (currentSpec==1 and not activePriSpec) or (currentSpec==2 and not activeSecSpec) then
		genabled = false;
		group.correctspec = false
	end
	

	if (genabled) then
		for row = 1, rows do
			for column = 1, columns do
				local iconID = (row-1)*columns + column;
				local iconName = groupName.."_Icon"..iconID;
				local icon = _G[iconName] or CreateFrame("Frame", iconName, group, "TellMeWhen_IconTemplate");
				local powerbarname = iconName.."_Power";
				local cooldownbarname = iconName.."_Cooldown";
				icon.powerbar = icon.powerbar or CreateFrame("StatusBar",powerbarname,icon)
				icon.cooldownbar = icon.cooldownbar or CreateFrame("StatusBar",cooldownbarname,icon)
				icon:SetID(iconID);
				icon:Show();
				if ( column > 1 ) then
					icon:SetPoint("TOPLEFT", _G[groupName.."_Icon"..(iconID-1)], "TOPRIGHT", TELLMEWHEN_ICONSPACING, 0);
				elseif ( row > 1 ) and ( column == 1 ) then
					icon:SetPoint("TOPLEFT", _G[groupName.."_Icon"..(iconID-columns)], "BOTTOMLEFT", 0, -TELLMEWHEN_ICONSPACING);
				elseif ( iconID == 1 ) then
					icon:SetPoint("TOPLEFT", group, "TOPLEFT");
				end
				TellMeWhen_Icon_Update(icon, groupID, iconID);
				if ( not genabled ) then
					TellMeWhen_Icon_ClearScripts(icon);
				end
				
				--this addes LibButtonFacade support I hope

 			--	if LBF then
 			--		LBF:Group("TellMeWhen",groupName):AddButton(icon);
 			--	end
			end
		end
		for iconID = rows*columns+1, TELLMEWHEN_MAXROWS*TELLMEWHEN_MAXROWS do
			local icon = _G[groupName.."_Icon"..iconID];
			if icon then
				icon:Hide();
				TellMeWhen_Icon_ClearScripts(icon);
			end
		end

		group:SetScale(scale);
		local p = TellMeWhen_Settings["Groups"][groupID]["Point"]
		if p.x then
			group:ClearAllPoints()
			group:SetPoint(p.point,UIParent,p.relativePoint,p.x,p.y)
		else
			p.point,_,p.relativePoint,p.x,p.y = group:GetPoint(1)
		end
		local lastIcon = groupName.."_Icon"..(rows*columns);
		resizeButton:SetPoint("BOTTOMRIGHT", lastIcon, "BOTTOMRIGHT", 3, -3);
		if ( locked ) then
			resizeButton:Hide();
		else
			resizeButton:Show();
		end
		
		TellMeWhen_Stance_Check(group)
	end -- Enabled

	if ( onlyInCombat and genabled and locked ) then
		group:RegisterEvent("PLAYER_REGEN_ENABLED");
		group:RegisterEvent("PLAYER_REGEN_DISABLED");
		group:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
		group:RegisterEvent("UPDATE_SHAPESHIFT_FORMS");
		group.combatcheck = true
		group:Hide();
	elseif (genabled and locked) then
		group:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
		group:RegisterEvent("UPDATE_SHAPESHIFT_FORMS");
		group.combatcheck = false
		group:Show();
	else
		group:UnregisterEvent("PLAYER_REGEN_ENABLED");
		group:UnregisterEvent("PLAYER_REGEN_DISABLED");
		group:UnregisterEvent("UPDATE_SHAPESHIFT_FORM");
		group:UnregisterEvent("UPDATE_SHAPESHIFT_FORMS");
		group.combatcheck = false
		if ( genabled ) then
			group:Show();
		else
			group:Hide();
		end
	end
	
	group:SetScript("OnEvent", TellMeWhen_Group_OnEvent)
end

function TellMeWhen_Stance_Check(group)
	local groupID = group:GetID()
	if not group.correctspec then
		return
	end
	local groupID = group:GetID()
	local index = GetShapeshiftForm()

	if pclass == "WARLOCK" and index == 2 then  --UGLY HACK FOR METAMORPHOSIS, IT IS INDEX 2 FOR SOME REASON
		index = 1
	end
	if pclass == "ROGUE" and index >= 2 then	--UGLY FIX FOR ROGUES, VANISH AND SHADOW DANCE RETURN 3...
		index = 1							--WHEN ACTIVE, VANISH RETURNS 2 WHEN SHADOW DANCE ISNT LEARNED.
	end
	if index > GetNumShapeshiftForms() then --MANY CLASSES RETURN AN INVALID NUMBER ON LOGIN, BUT NOT ANYMORE!
		index = 0
	end
	if index == 0 then
		if not TellMeWhen_Settings["Groups"][groupID]["Stance"][0] then
			if group.combatcheck then
				if UnitAffectingCombat("player") then
					group:Show();
				elseif not UnitAffectingCombat("player") then
					group:Hide();
				end
			else
				group:Show()
			end
		else
			group:Hide();
		end
	elseif index ~= nil then
		local texture, name, isActive, isCastable = GetShapeshiftFormInfo(index)
		if not name then error("Uh oh! Something happened to the stance checks! Please send this error in Cybeloras' direction, as well as what you were doing at the time: " .. index .. " " .. pclass) return end
		local _,_,ID = string.find(GetSpellLink(name), ":(%d+)")
		for k,v in pairs(TellMeWhen_CurrentStances) do
			if TellMeWhen_CurrentStances[k] == tonumber(ID) then
				if not TellMeWhen_Settings["Groups"][groupID]["Stance"][k] then
					if group.combatcheck then
						if UnitAffectingCombat("player") then
							group:Show();
						elseif not UnitAffectingCombat("player") then
							group:Hide();
						end
					else
						group:Show()
					end
				else
					group:Hide();
				end
			end
		end
	end
end


-- -------------
-- ICON FUNCTION
-- -------------
function TellMeWhen_Icon_Bars_Update(icon, groupID, iconID)
	local iconSettings = TellMeWhen_Settings["Groups"][groupID]["Icons"][iconID];
	if icon.ShowPBar or icon.ShowCBar then
		local groupName = "TellMeWhen_Group"..groupID;
		local iconName = groupName.."_Icon"..iconID;
		local genabled = TellMeWhen_Settings["Groups"][groupID]["Enabled"];
		local locked = TellMeWhen_Settings["Locked"];
		local onlyInCombat = TellMeWhen_Settings["Groups"][groupID]["OnlyInCombat"];
		local width, height = icon:GetSize()
		local scale = TellMeWhen_Settings["Groups"][groupID]["Scale"];
		if not TellMeWhen_Settings["Texture"] then
			TellMeWhen_Settings["Texture"] = "Interface\\TargetingFrame\\UI-StatusBar";
		end
		if not TellMeWhen_Settings["TextureName"] then
			TellMeWhen_Settings["TextureName"] = "Blizzard";
		end
		local tex = TellMeWhen_Settings["Texture"]
		if icon.ShowPBar then
			local cost,_,powerType = select(4,GetSpellInfo(icon.namefirst));
			if cost == nil then cost = 0 end
			local powerbarname = iconName.."_Power";
			if not icon.powerbar then
				icon.powerbar = CreateFrame("StatusBar",powerbarname,icon)
			end
			icon.powerbar:SetSize(width*0.9, (height / 2)*0.87);
			icon.powerbar:SetPoint("TOP",icon,"TOP",0,-1.5);
			if cost then
				icon.powerbar:SetMinMaxValues(0, cost);
			end
			if not icon.powerbar.texture then
				icon.powerbar.texture = icon.powerbar:CreateTexture()
			end
			icon.powerbar.texture:SetTexture(tex);
			if powerType then
				local colorinfo = PowerBarColor[powerType];
				icon.powerbar.texture:SetVertexColor(colorinfo.r, colorinfo.g, colorinfo.b, 0.9);
			end
			icon.powerbar:SetStatusBarTexture(icon.powerbar.texture);
			icon.powerbar:SetFrameLevel(icon:GetFrameLevel() + 2);
		end
		if icon.ShowCBar then
			local cooldownbarname = iconName.."_Cooldown";
			icon.cooldownbar = icon.cooldownbar or CreateFrame("StatusBar",cooldownbarname,icon);
			icon.cooldownbar:SetSize(width*0.9, (height / 2)*0.87);
			icon.cooldownbar:SetPoint("BOTTOM",icon,"BOTTOM",0,1.5);
			icon.cooldownbar.texture = icon.cooldownbar.texture or icon.cooldownbar:CreateTexture();
			icon.cooldownbar.texture:SetTexture(tex);
			icon.cooldownbar:SetStatusBarTexture(icon.cooldownbar.texture);
			icon.cooldownbar:SetFrameLevel(icon:GetFrameLevel() + 2);
			icon.cooldownbar:SetMinMaxValues(0,  1)
		end
	end
	if not icon.ShowPBar then
		icon.powerbar:Hide();
	else
		icon.powerbar:Show()
	end
	if not icon.ShowCBar then
		icon.cooldownbar:Hide();
	else
		icon.cooldownbar:Show()
	end
end

function TellMeWhen_PetEvent()
	for groupID = 1, TELLMEWHEN_MAXGROUPS do
		local currentSpec = GetActiveTalentGroup();
		local activePriSpec = TellMeWhen_Settings["Groups"][groupID]["PrimarySpec"];
		local activeSecSpec = TellMeWhen_Settings["Groups"][groupID]["SecondarySpec"];
		if (currentSpec==1 and not activePriSpec) or (currentSpec==2 and not activeSecSpec) then
			return
		end
		local rows = TellMeWhen_Settings["Groups"][groupID]["Rows"];
		local columns = TellMeWhen_Settings["Groups"][groupID]["Columns"];
		local genabled = TellMeWhen_Settings["Groups"][groupID]["Enabled"];
		local groupName = "TellMeWhen_Group"..groupID;
		if (genabled) then
			for row = 1, rows do
				for column = 1, columns do
					local iconID = (row-1)*columns + column;
					local iconName = groupName.."_Icon"..iconID;
					local icon = _G[iconName] or CreateFrame("Frame", iconName, group, "TellMeWhen_IconTemplate");
					if ( icon.iconType == "cooldown" ) or ( icon.iconType == "buff" ) then
						if icon.texture:GetTexture() == "Interface\\Icons\\INV_Misc_QuestionMark" then
							if icon.namefirst then
								local tex = GetSpellTexture(icon.namefirst)
								if tex then
									icon.texture:SetTexture(tex)
								else
									icon.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
								end
							end
						end
					end
				end
			end
		end
	end
end

local SM = LibStub("LibSharedMedia-3.0")
function TellMeWhen_Icon_Update(icon, groupID, iconID)
	local iconSettings 		= TellMeWhen_Settings["Groups"][groupID]["Icons"][iconID];
	local Enabled 			= iconSettings.Enabled;
	icon.iconType 			= iconSettings.Type;
	local CooldownType 		= iconSettings.CooldownType;
	icon.CooldownShowWhen 	= iconSettings.CooldownShowWhen;
	icon.BuffShowWhen 		= iconSettings.BuffShowWhen;
	local Conditions 		= iconSettings.Conditions;
	local ConditionPresent 	= false;
	icon.Name 				= iconSettings.Name;
	icon.Unit 				= iconSettings.Unit;
	icon.UnitReact 			= iconSettings.UnitReact;
	icon.ShowTimer 			= iconSettings.ShowTimer;
	icon.ShowPBar 			= iconSettings.ShowPBar;
	icon.ShowCBar 			= iconSettings.ShowCBar;
	icon.InvertBars 		= iconSettings.InvertBars;
	icon.OnlyMine 			= iconSettings.OnlyMine;
	icon.BuffOrDebuff 		= iconSettings.BuffOrDebuff;
	icon.WpnEnchantType 	= iconSettings.WpnEnchantType;
	icon.RangeCheck			= iconSettings.RangeCheck;
	icon.ManaCheck			= iconSettings.ManaCheck;
	icon.baseAlpha 			= iconSettings.Alpha;
	icon.StackMin			= iconSettings.StackMin or 0;
	icon.StackMax			= iconSettings.StackMax or 1000;
	
	TellMeWhen_Settings["Interval"] = TellMeWhen_Settings["Interval"] or TELLMEWHEN_UPDATE_INTERVAL
	icon.updateTimer = TellMeWhen_Settings["Interval"];

	icon.texture = _G[icon:GetName().."Texture"];
	icon.countText = _G[icon:GetName().."Count"];
	icon.Cooldown = _G[icon:GetName().."Cooldown"];
	icon.Cooldown:SetFrameLevel(icon:GetFrameLevel() + 1)
	icon.Cooldown:SetDrawEdge(TellMeWhen_Settings["DrawEdge"])

	icon:UnregisterEvent("ACTIONBAR_UPDATE_STATE");
	icon:UnregisterEvent("ACTIONBAR_UPDATE_USABLE");
	icon:UnregisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
	--icon:UnregisterEvent("SPELL_UPDATE_USABLE");
	icon:UnregisterEvent("PLAYER_TARGET_CHANGED");
	icon:UnregisterEvent("PLAYER_FOCUS_CHANGED");
	icon:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	icon:UnregisterEvent("UNIT_INVENTORY_CHANGED");
	icon:UnregisterEvent("BAG_UPDATE_COOLDOWN");
	icon:UnregisterEvent("UNIT_AURA");
	icon:UnregisterEvent("PLAYER_TOTEM_UPDATE");
	icon:UnregisterEvent("PLAYER_REGEN_ENABLED");
	icon:UnregisterEvent("PLAYER_REGEN_DISABLED");
	if ( TellMeWhen_Settings["Locked"] and not Enabled) then
		TellMeWhen_Icon_ClearScripts(icon);
	else

		-- used by both cooldown and reactive icons
		if ( icon.CooldownShowWhen == "usable" ) then
			icon.usableAlpha = 1;
			icon.unusableAlpha = 0;
		elseif ( icon.CooldownShowWhen == "unusable" ) then
			icon.usableAlpha = 0;
			icon.unusableAlpha = 1;
		elseif ( icon.CooldownShowWhen == "always") then
			icon.usableAlpha = 1;
			icon.unusableAlpha = 1;
		else
			error("Alpha not assigned: "..icon.Name);
			icon.usableAlpha = 1;
			icon.unusableAlpha = 1;
		end
		-- used by both buff/debuff and wpnenchant icons
		if ( icon.BuffShowWhen == "present" ) then
			icon.presentAlpha = 1;
			icon.absentAlpha = 0;
		elseif ( icon.BuffShowWhen == "absent" ) then
			icon.presentAlpha = 0;
			icon.absentAlpha = 1;
		elseif ( icon.BuffShowWhen == "always") then
			icon.presentAlpha = 1;
			icon.absentAlpha = 1;
			--SendChatMessage("Alpha always assigned to: "..icon.Name);
		else
			error("Alpha not assigned: "..icon.Name);
			icon.presentAlpha = 1;
			icon.absentAlpha = 1;
		end


		if ( icon.iconType == "cooldown" ) then
-- --------------				
-- SPELL COOLDOWN
-- --------------			
			if ( CooldownType == "spell" ) then
				icon.namefirst = TellMeWhen_GetSpellNames(icon.Name,1)
				icon.namename = TellMeWhen_GetSpellNames(icon.Name,1,true)
				
				if ( GetSpellCooldown(icon.namefirst) ) then
					icon.texture:SetTexture(GetSpellTexture(icon.namefirst));
					icon:SetScript("OnUpdate", TellMeWhen_Icon_SpellCooldown_OnUpdate);
				else
					TellMeWhen_Icon_ClearScripts(icon);
					icon.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
				end
				TellMeWhen_Icon_Bars_Update(icon, groupID, iconID)
				if (icon.ShowTimer) then
					icon:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
					icon:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
					icon:SetScript("OnEvent", TellMeWhen_Icon_SpellCooldown_OnEvent);
				else
					icon:SetScript("OnEvent", nil);
				end
				TellMeWhen_Icon_StatusCheck(icon, icon.iconType, CooldownType);
-- --------------				
-- ITEM COOLDOWN
-- --------------					
			elseif ( CooldownType == "item" ) then
				icon.namefirst = TellMeWhen_GetItemIDs(icon.Name,1)
				icon.ShowPBar = false;
				icon.powerbar:Hide();
				TellMeWhen_Icon_Bars_Update(icon, groupID, iconID)
				if not icon.namefirst then
					return  --silently error, this will be recalled again and everything should be fine.
				end
				local itemName, itemLink, _, _, _, _, _, _, _, itemTexture = GetItemInfo(icon.namefirst)
					icon:SetScript("OnUpdate", TellMeWhen_Icon_ItemCooldown_OnUpdate)
			--	end
				if ( itemName ) then
					icon.texture:SetTexture(itemTexture);
					if (icon.ShowTimer) then
						icon:RegisterEvent("BAG_UPDATE_COOLDOWN");
						icon:SetScript("OnEvent", TellMeWhen_Icon_ItemCooldown_OnEvent);
					else
						icon:SetScript("OnEvent", nil);
					end
				else
					TellMeWhen_Icon_ClearScripts(icon);
					icon.learnedTexture = false;
					icon.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
				end
			end
			icon.Cooldown:SetReverse(false);
			TellMeWhen_Icon_StatusCheck(icon, icon.iconType, CooldownType);
-- --------------				
-- BUFF
-- --------------
		elseif ( icon.iconType == "buff" ) then
			icon.namefirst = TellMeWhen_GetSpellNames(icon.Name,1)
			icon.namename = TellMeWhen_GetSpellNames(icon.Name,1,true)
			icon.namelist = TellMeWhen_GetSpellNames(icon.Name)
			TellMeWhen_Icon_Bars_Update(icon, groupID, iconID)
			icon:SetScript("OnUpdate",TellMeWhen_Icon_Buff_OnUpdate)
			local name = icon.namefirst
			
			if ( icon.Name == "" ) then
				icon.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
			elseif ( GetSpellTexture(name) ) then
				icon.texture:SetTexture(GetSpellTexture(name));
			elseif ( not icon.learnedTexture ) then
				icon.texture:SetTexture("Interface\\Icons\\INV_Misc_PocketWatch_01");
			end
			icon.Cooldown:SetReverse(true);
			TellMeWhen_Icon_StatusCheck(icon, icon.iconType);
-- --------------				
-- REACTIVE
-- --------------
		elseif ( icon.iconType == "reactive" ) then
			icon.namefirst = TellMeWhen_GetSpellNames(icon.Name,1)
			icon.namename = TellMeWhen_GetSpellNames(icon.Name,1,true)

			TellMeWhen_Icon_Bars_Update(icon, groupID, iconID)
			if ( GetSpellTexture(icon.namefirst) ) then
				icon.texture:SetTexture(GetSpellTexture(icon.namefirst));
				icon:SetScript("OnUpdate", TellMeWhen_Icon_Reactive_OnUpdate);
			else
				TellMeWhen_Icon_ClearScripts(icon);
				icon.learnedTexture = false;
				icon.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
			end
			TellMeWhen_Icon_StatusCheck(icon, icon.iconType);

		
-- --------------				
-- WEP ENCHANT
-- --------------	
		elseif ( icon.iconType == "wpnenchant" ) then	
			icon.namefirst = TellMeWhen_GetSpellNames(icon.Name,1)
			
			icon.ShowPBar = false;
			icon.ShowCBar = false;			
			TellMeWhen_Icon_Bars_Update(icon, groupID, iconID)
			icon:RegisterEvent("UNIT_INVENTORY_CHANGED");
			local slotID;
			if ( icon.WpnEnchantType == "mainhand" ) then
				slotID = GetInventorySlotInfo("MainHandSlot");
			elseif ( icon.WpnEnchantType == "offhand" ) then
				slotID = GetInventorySlotInfo("SecondaryHandSlot");
			end
			local wpnTexture = GetInventoryItemTexture("player", slotID);
			if ( wpnTexture ) then
				icon.texture:SetTexture(wpnTexture);
				icon:SetScript("OnEvent", TellMeWhen_Icon_WpnEnchant_OnEvent);
				icon:SetScript("OnUpdate", TellMeWhen_Icon_WpnEnchant_OnUpdate);
			else
				TellMeWhen_Icon_ClearScripts(icon);
				icon.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
			end
	
-- --------------				
-- TOTEM
-- --------------	
		elseif ( icon.iconType == "totem" ) then
			icon.namefirst = TellMeWhen_GetSpellNames(icon.Name,1)
			icon.namename = TellMeWhen_GetSpellNames(icon.Name,1,true)
			icon.namelist = TellMeWhen_GetSpellNames(icon.Name)
		
			icon.ShowPBar = false;
			icon.ShowCBar = false;
			TellMeWhen_Icon_Bars_Update(icon, groupID, iconID)
		
			icon:RegisterEvent("PLAYER_TOTEM_UPDATE");
			icon:SetScript("OnEvent", TellMeWhen_Icon_Totem_OnEvent);
			icon:SetScript("OnUpdate", TellMeWhen_Icon_Totem_OnEvent);
			TellMeWhen_Icon_Totem_OnEvent(icon);
			if ( icon.Name == "" ) then
				icon.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
				icon.learnedTexture = false;
			elseif ( GetSpellTexture(icon.namefirst) ) then
				icon.texture:SetTexture(GetSpellTexture(icon.namefirst));
			elseif ( not icon.learnedTexture ) then
				icon.texture:SetTexture("Interface\\Icons\\INV_Misc_PocketWatch_01");
			end

		else
			icon.ShowPBar = false;
			icon.ShowCBar = false;
			TellMeWhen_Icon_Bars_Update(icon, groupID, iconID)
			TellMeWhen_Icon_ClearScripts(icon);
			if ( icon.Name ~= "" ) then
				icon.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
			else
				icon.texture:SetTexture(nil);
			end
		end
	end -- Enabled CHECK

	icon.countText:Hide();
	icon.Cooldown:Hide();

	if ( Enabled ) then
		icon:SetAlpha(1.0);
	else
		icon:SetAlpha(0.4);
		TellMeWhen_Icon_ClearScripts(icon);
	end

	icon:Show();
	if ( TellMeWhen_Settings["Locked"] ) then
		icon:EnableMouse(0);
		if ( not Enabled ) then
			icon:Hide();
			icon.powerbar:Hide();
			icon.cooldownbar:Hide();
		elseif (icon.Name == "") and ( icon.iconType ~= "wpnenchant" ) then
			icon:Hide();
		end
		icon.powerbar:SetValue(0);
		icon.cooldownbar:SetValue(0);
		icon.powerbar:SetAlpha(.9)

	--	
	else
		if not icon.cooldownbar.texture then
			icon.cooldownbar.texture = icon.cooldownbar:CreateTexture()
		end
		if not icon.powerbar.texture then
			icon.powerbar.texture = icon.powerbar:CreateTexture()
		end
		icon.cooldownbar:SetMinMaxValues(0,  1)
		icon.cooldownbar:SetValue(2000000)
		icon.cooldownbar:SetStatusBarColor(0, 1, 0, 0.5)
		icon.cooldownbar.texture:SetTexCoord(0, 1, 0, 1)
		icon.powerbar:SetValue(20000)
		icon.powerbar:SetAlpha(.5)
		icon.powerbar.texture:SetTexCoord(0, 1, 0, 1)
		icon:EnableMouse(1);
		icon.texture:SetVertexColor(1, 1, 1, 1);
		TellMeWhen_Icon_ClearScripts(icon);
	end
end

function TellMeWhen_Icon_ClearScripts(icon)
	icon:SetScript("OnEvent", nil);
	icon:SetScript("OnUpdate", nil);
end

function TellMeWhen_GetGCD()
	return select(2,GetSpellCooldown(defaultSpells[pclass]));
end

function TellMeWhen_Icon_StatusCheck(icon, iconType, CooldownType)
	-- this function is so OnEvent-based icons can do a check when the addon is locked
	-- the 1s trick it into thinking that it has been a long time since the last onupdate and so it will run the whole function.
	if ( iconType == "reactive" ) then
		TellMeWhen_Icon_Reactive_OnUpdate(icon,1)
	elseif ( iconType == "buff" ) then
		TellMeWhen_Icon_Buff_OnUpdate(icon,1);
	elseif ( iconType == "cooldown" ) then
		if (CooldownType == "spell") then
			TellMeWhen_Icon_SpellCooldown_OnUpdate(icon,1);
		elseif (CooldownType == "item") then
			TellMeWhen_Icon_ItemCooldown_OnUpdate(icon,1);
		end
	end
end

function TellMeWhen_Icon_SpellCooldown_OnEvent(icon,event)
	local startTime, timeLeft = GetSpellCooldown(icon.namefirst);
	if ( timeLeft ) then
		CooldownFrame_SetTimer(icon.Cooldown, startTime, timeLeft, 1);
	end
end
	
local percentcomplete = 1
function TellMeWhen_Icon_SpellCooldown_OnUpdate(icon, elapsed)
	
	icon.updateTimer = icon.updateTimer - elapsed;
	local name = icon.namefirst
	local startTime, timeLeft = GetSpellCooldown(name);
	if timeLeft then
		if icon.ShowPBar then
			icon.powerbar:SetAlpha(1)
			local cost,_,powerType = select(4,GetSpellInfo(name))
			if cost == nil then cost = 0 end
			icon.powerbar:SetMinMaxValues(0, cost)
			if not icon.InvertBars then
				icon.powerbar:SetValue(cost - UnitPower("player",powerType))
				icon.powerbar.texture:SetTexCoord(0, max(0,  min(((cost - UnitPower("player",powerType)) / cost),  1)), 0, 1) --more cheats
			else
				icon.powerbar:SetValue(UnitPower("player",powerType))
				icon.powerbar.texture:SetTexCoord(0, max(0,min((UnitPower("player",powerType) / cost),1)), 0, 1)			--more cheats
			end
		end

		if icon.ShowCBar then
			if not icon.InvertBars then
				if (timeLeft == 0) then
					icon.cooldownbar:SetMinMaxValues(0, 1)
					icon.cooldownbar:SetValue(0)
				else
					percentcomplete = (((GetTime() - startTime) / timeLeft))
					icon.cooldownbar:SetMinMaxValues(0,  timeLeft)
					icon.cooldownbar:SetValue(timeLeft - (GetTime() - startTime))
					icon.cooldownbar.texture:SetTexCoord(0, min((1-percentcomplete),1), 0, 1)
					icon.cooldownbar:SetStatusBarColor(
						(co.r*percentcomplete) + (st.r * (1-percentcomplete)),
						(co.g*percentcomplete) + (st.g * (1-percentcomplete)),
						(co.b*percentcomplete) + (st.b * (1-percentcomplete)),
						(co.a*percentcomplete) + (st.a * (1-percentcomplete))
					)
				end
			else
				--inverted
				if (timeLeft == 0) then
					icon.cooldownbar:SetMinMaxValues(0,  1)
					icon.cooldownbar:SetValue(1)
					icon.cooldownbar:SetStatusBarColor(co.r, co.g, co.b, co.a)
					icon.cooldownbar.texture:SetTexCoord(0, 1, 0, 1)
				else
					percentcomplete = (((GetTime() - startTime) / timeLeft))
					icon.cooldownbar:SetMinMaxValues(0,  timeLeft)
					icon.cooldownbar:SetValue(GetTime() - startTime)
					icon.cooldownbar.texture:SetTexCoord(0, min((percentcomplete),1), 0, 1)
					icon.cooldownbar:SetStatusBarColor(
						(co.r*percentcomplete) + (st.r * (1-percentcomplete)),
						(co.g*percentcomplete) + (st.g * (1-percentcomplete)),
						(co.b*percentcomplete) + (st.b * (1-percentcomplete)),
						(co.a*percentcomplete) + (st.a * (1-percentcomplete))
					)
				end
			end
		end
		if ( icon.updateTimer <= 0 ) then
			icon.updateTimer = TellMeWhen_Settings["Interval"];
			if ( not TellMeWhen_Icon_ConditionCheck(icon) ) then
				icon:SetAlpha(0);
				return;
			end
			local isEnemy = UnitIsEnemy("player", "target")
			local react = UnitReaction("player", "target") or 5
			if (isEnemy) or (react <= 4) then
				isEnemy = 1
			else
				isEnemy = 0
			end
			if (icon.UnitReact == 0) or (icon.UnitReact == isEnemy) or (icon.UnitReact == 2 and isEnemy == 0 ) then
				
				local inrange = IsSpellInRange(icon.namename, "target");
				local _, nomana = IsUsableSpell(name);
				local OnGCD = TellMeWhen_GetGCD() == timeLeft and timeLeft > 0;
				--name, rank, icon, powerCost,  isFunnel, powerType, castingTime,  minRange, maxRange
				local minRange,maxRange = select(8,GetSpellInfo(name));
				if ( not icon.RangeCheck or not maxRange or inrange == nil ) then
					inrange = 1;
				end
				if not icon.ManaCheck then
					nomana = nil
				end
				if ( (timeLeft == 0 or OnGCD) and inrange == 1 and not nomana) then
					icon.texture:SetVertexColor(1, 1, 1, 1);
					TellMeWhen_SetIconAlpha(icon, icon.usableAlpha);
				elseif ( icon.usableAlpha == 1 and  (timeLeft == 0 or OnGCD)) then
					--gray if not inrange or if nomana
					icon.texture:SetVertexColor(0.5, 0.5, 0.5, 1);
					TellMeWhen_SetIconAlpha(icon, icon.usableAlpha);
				else
					icon.texture:SetVertexColor(1, 1, 1, 1);
					TellMeWhen_SetIconAlpha(icon, icon.unusableAlpha);
				end
			else
				icon:SetAlpha(0)
			end
		end
	end
end

function TellMeWhen_Icon_ItemCooldown_OnEvent(icon)
	local startTime, timeLeft, enable = GetItemCooldown(icon.namefirst);
	if ( timeLeft ) then
		CooldownFrame_SetTimer(icon.Cooldown, startTime, timeLeft, 1);
	end
end

function TellMeWhen_Icon_ItemCooldown_OnUpdate(icon, elapsed)
	icon.updateTimer = icon.updateTimer - elapsed;
	local startTime, timeLeft = GetItemCooldown(icon.namefirst);
	if icon.ShowCBar then
		if not icon.InvertBars then
			if (timeLeft == 0) then
				icon.cooldownbar:SetMinMaxValues(0,  1)
				icon.cooldownbar:SetValue(0)
			else
				percentcomplete = ((GetTime() - startTime) / timeLeft)
				icon.cooldownbar:SetMinMaxValues(0,  timeLeft)
				icon.cooldownbar:SetValue(timeLeft - (GetTime() - startTime))
				icon.cooldownbar.texture:SetTexCoord(0, min((1-percentcomplete),1), 0, 1)
				icon.cooldownbar:SetStatusBarColor(
					(co.r*percentcomplete) + (st.r * (1-percentcomplete)),
					(co.g*percentcomplete) + (st.g * (1-percentcomplete)),
					(co.b*percentcomplete) + (st.b * (1-percentcomplete)),
					(co.a*percentcomplete) + (st.a * (1-percentcomplete))
				)
			end
		else
			--inverted
			if (timeLeft == 0) then
				icon.cooldownbar:SetMinMaxValues(0,  1)
				icon.cooldownbar:SetValue(1)
				icon.cooldownbar:SetStatusBarColor(co.r, co.g, co.b, co.a)
				icon.cooldownbar.texture:SetTexCoord(0, 1, 0, 1)
			else
				percentcomplete = (((GetTime() - startTime) / timeLeft))
				icon.cooldownbar:SetMinMaxValues(0,  timeLeft)
				icon.cooldownbar:SetValue(GetTime() - startTime)
				icon.cooldownbar.texture:SetTexCoord(0, min(percentcomplete,1), 0, 1)
				icon.cooldownbar:SetStatusBarColor(
					(co.r*percentcomplete) + (st.r * (1-percentcomplete)),
					(co.g*percentcomplete) + (st.g * (1-percentcomplete)),
					(co.b*percentcomplete) + (st.b * (1-percentcomplete)),
					(co.a*percentcomplete) + (st.a * (1-percentcomplete))
				)
			end
		end
	end
	if ( icon.updateTimer <= 0 ) and timeLeft then
		icon.updateTimer = TellMeWhen_Settings["Interval"];
		if ( not TellMeWhen_Icon_ConditionCheck(icon) ) then
			icon:SetAlpha(0);
			return;
		end
		local isEnemy = UnitIsEnemy("player", "target")
		local react = UnitReaction("player", "target") or 5
		if (isEnemy) or (react <= 4) then
			isEnemy = 1
		else
			isEnemy = 0
		end
		if (icon.UnitReact == 0) or (icon.UnitReact == isEnemy) or (icon.UnitReact == 2 and isEnemy == 0 ) then
			if ( timeLeft == 0 or TellMeWhen_GetGCD() == timeLeft ) then
				TellMeWhen_SetIconAlpha(icon, icon.usableAlpha);
			elseif ( timeLeft > 0 and TellMeWhen_GetGCD() ~= timeLeft ) then
				TellMeWhen_SetIconAlpha(icon, icon.unusableAlpha);
			end
		else
			icon:SetAlpha(0)
		end
	end
end

function TellMeWhen_Icon_Buff_OnUpdate(icon, elapsed)
	local auraNames = icon.namelist;
	local isEnemy = UnitIsEnemy("player", icon.Unit)
	local react = UnitReaction("player", icon.Unit) or 5
	if (isEnemy) or (react <= 4) then
		isEnemy = 1
	else
		isEnemy = 0
	end
	if(icon.UnitReact == 0) or (icon.UnitReact == isEnemy) or (icon.UnitReact == 2 and isEnemy == 0 ) then
		
		local maxExpirationTime = 0;
		local processedBuffInAuraNames = false;
		local i, iName;
		local filter = icon.BuffOrDebuff;
		if icon.OnlyMine then filter = filter.."|PLAYER" end
		if ( not TellMeWhen_Icon_ConditionCheck(icon) ) then
			icon:SetAlpha(0);
			return;
		end
		for i, iName in ipairs(auraNames) do
			if tonumber(iName) then -- UnitAura requires a spell name, numbers are treated as an aura index on the unit instead of IDs.
				iName = GetSpellInfo(iName)
			end
			local buffName, _, iconTexture, count, _, duration, expirationTime = UnitAura(icon.Unit, iName, nil, filter)
			if tonumber(count) and not (icon.StackMin <= count and count <= icon.StackMax) then
				icon:SetAlpha(0);
				return;
			end
			if icon.ShowPBar then
				local cost,_,powerType = select(4,GetSpellInfo(iName))
				if cost == nil then cost = 0 end
				icon.powerbar:SetMinMaxValues(0, cost);
				if not icon.InvertBars then
					icon.powerbar:SetValue(cost - UnitPower("player",powerType))
					icon.powerbar.texture:SetTexCoord(0, max(0,min(((cost - UnitPower("player",powerType)) / cost),1)), 0, 1) --more cheats
				else
					icon.powerbar:SetValue(UnitPower("player",powerType))
					icon.powerbar.texture:SetTexCoord(0, max(0,min((UnitPower("player",powerType) / cost),1)), 0, 1)				--more cheats
				end
			end
			local startTime, timeLeft = GetSpellCooldown(iName)
			if ( buffName ) then
				if ( icon.texture:GetTexture() ~= iconTexture) then
					icon.texture:SetTexture(iconTexture);
					icon.learnedTexture = true;
				end
				if (icon.presentAlpha) then
					TellMeWhen_SetIconAlpha(icon,icon.presentAlpha);
				end
				icon.texture:SetVertexColor(1, 1, 1, 1);
				if ( count > 1 ) then
					icon.countText:SetText(count);
					icon.countText:Show();
				else
					icon.countText:Hide();
				end
				if ( icon.ShowTimer and not UnitIsDead(icon.Unit)) then
					CooldownFrame_SetTimer(icon.Cooldown, expirationTime - duration, duration, 1);
				end
				processedBuffInAuraNames = true;
				if icon.ShowCBar then
					if not icon.InvertBars then
						if (duration == nil or duration == 0) then
							icon.cooldownbar:SetMinMaxValues(0,  1)
							icon.cooldownbar:SetValue(0)
						else
							percentcomplete = (( duration - (expirationTime - GetTime()) )/duration)
							icon.cooldownbar:SetMinMaxValues(0,  duration)
							icon.cooldownbar:SetValue(expirationTime - GetTime())
							icon.cooldownbar.texture:SetTexCoord(0, min((1-(percentcomplete)),1), 0, 1)
							icon.cooldownbar:SetStatusBarColor(
								(co.r*percentcomplete) + (st.r * (1-percentcomplete)),
								(co.g*percentcomplete) + (st.g * (1-percentcomplete)),
								(co.b*percentcomplete) + (st.b * (1-percentcomplete)),
								(co.a*percentcomplete) + (st.a * (1-percentcomplete))
							)
						end
					else
						--inverted
						if (duration == nil or duration == 0) then
							percentcomplete = 1
							icon.cooldownbar:SetMinMaxValues(0,  1)
							icon.cooldownbar:SetValue(1)
							icon.cooldownbar:SetStatusBarColor(co.r, co.g, co.b, co.a)
							icon.cooldownbar.texture:SetTexCoord(0, 1, 0, 1)
						else
							percentcomplete = (( duration - (expirationTime - GetTime()) )/duration)
							icon.cooldownbar:SetMinMaxValues(0,  duration)
							icon.cooldownbar:SetValue(duration - (expirationTime - GetTime()))
							icon.cooldownbar.texture:SetTexCoord(0, min((percentcomplete),1), 0, 1)
							icon.cooldownbar:SetStatusBarColor(
								(co.r*percentcomplete) + (st.r * (1-percentcomplete)),
								(co.g*percentcomplete) + (st.g * (1-percentcomplete)),
								(co.b*percentcomplete) + (st.b * (1-percentcomplete)),
								(co.a*percentcomplete) + (st.a * (1-percentcomplete))
							)
						end
					end
				end
			end
		end						
								
		if (processedBuffInAuraNames) then
			 return;
		end
		if not buffName and not processedBuffInAuraNames then
			icon.cooldownbar:SetMinMaxValues(0,  1)
			icon.cooldownbar:SetValue(0)
		end
		if (icon.absentAlpha) then
			TellMeWhen_SetIconAlpha(icon,icon.absentAlpha);
		end
		if ( icon.presentAlpha == 1 ) and ( icon.absentAlpha == 1) then
			icon.texture:SetVertexColor(1, 0.35, 0.35, 1);
		end

		icon.countText:Hide();
		if ( icon.ShowTimer  ) then
			CooldownFrame_SetTimer(icon.Cooldown, 0, 0, 0);
		end
	else
		icon:SetAlpha(0);
		CooldownFrame_SetTimer(icon.Cooldown, 0, 0, 0);
	end
end

function TellMeWhen_Icon_Reactive_OnUpdate(icon,elapsed)
	icon.updateTimer = icon.updateTimer - elapsed;
	local name = icon.namefirst
	local startTime, timeLeft = GetSpellCooldown(name);
	if timeLeft then
		if icon.ShowPBar then
			local cost,_,powerType = select(4,GetSpellInfo(name))
			if cost == nil then cost = 0 end
			icon.powerbar:SetMinMaxValues(0, cost);
			if not icon.InvertBars then
				icon.powerbar:SetValue(cost - UnitPower("player",powerType))
				icon.powerbar.texture:SetTexCoord(0, max(0,min(((cost - UnitPower("player",powerType)) / cost),1)), 0, 1) --more cheats
			else
				icon.powerbar:SetValue(UnitPower("player",powerType))
				icon.powerbar.texture:SetTexCoord(0, max(0,min((UnitPower("player",powerType) / cost),1)), 0, 1)			--more cheats
			end
		end

		if icon.ShowCBar then
			if not icon.InvertBars then
				if (timeLeft == 0) then
					icon.cooldownbar:SetMinMaxValues(0,  1)
					icon.cooldownbar:SetValue(0)
				else
					percentcomplete = ((GetTime() - startTime) / timeLeft)
					icon.cooldownbar:SetMinMaxValues(0,  timeLeft)
					icon.cooldownbar:SetValue(timeLeft - (GetTime() - startTime))
					icon.cooldownbar.texture:SetTexCoord(0, min((1-percentcomplete),1), 0, 1)
					icon.cooldownbar:SetStatusBarColor(
							(co.r*percentcomplete) + (st.r * (1-percentcomplete)),
							(co.g*percentcomplete) + (st.g * (1-percentcomplete)),
							(co.b*percentcomplete) + (st.b * (1-percentcomplete)),
							(co.a*percentcomplete) + (st.a * (1-percentcomplete))
						)
				end
			else
				--inverted
				if (timeLeft == 0) then
					icon.cooldownbar:SetMinMaxValues(0,  1)
					icon.cooldownbar:SetValue(1)
					icon.cooldownbar:SetStatusBarColor(co.r, co.g, co.b, co.a)
					icon.cooldownbar.texture:SetTexCoord(0, 1, 0, 1)
				else
					percentcomplete = ((GetTime() - startTime) / timeLeft)
					icon.cooldownbar:SetMinMaxValues(0,  timeLeft)
					icon.cooldownbar:SetValue(GetTime() - startTime)
					icon.cooldownbar.texture:SetTexCoord(0, min((percentcomplete),1), 0, 1)
					icon.cooldownbar:SetStatusBarColor(
							(co.r*percentcomplete) + (st.r * (1-percentcomplete)),
							(co.g*percentcomplete) + (st.g * (1-percentcomplete)),
							(co.b*percentcomplete) + (st.b * (1-percentcomplete)),
							(co.a*percentcomplete) + (st.a * (1-percentcomplete))
						)
				end
			end
		end
		if ( icon.updateTimer <= 0 ) and timeLeft then
			icon.updateTimer = TellMeWhen_Settings["Interval"];
			if ( not TellMeWhen_Icon_ConditionCheck(icon) ) then
				icon:SetAlpha(0);
				return;
			end
			local isEnemy = UnitIsEnemy("player", "target")
			local react = UnitReaction("player", "target") or 5
			if (isEnemy) or (react <= 4) then
				isEnemy = 1
			else
				isEnemy = 0
			end
			
			if (icon.UnitReact == 0) or (icon.UnitReact == isEnemy) or (icon.UnitReact == 2 and isEnemy == 0 ) then
				local usable, nomana = IsUsableSpell(name);
				local inrange = IsSpellInRange(icon.namename, "target");
				local OnGCD = TellMeWhen_GetGCD() == timeLeft and timeLeft > 0;
				if ( not icon.RangeCheck or inrange == nil ) then
					inrange = 1
				end
				if not icon.ManaCheck then
					nomana = nil
				end
				if ( usable ) then
					if( inrange == 1 and not nomana  ) then
						icon.texture:SetVertexColor(1,1,1,1)
						TellMeWhen_SetIconAlpha(icon,icon.usableAlpha)
					elseif ( not inrange or nomana ) then
						icon.texture:SetVertexColor(.35,.35,.35,1)
						TellMeWhen_SetIconAlpha(icon,icon.usableAlpha)
					else
						icon.texture:SetVertexColor(1,1,1,1)
						TellMeWhen_SetIconAlpha(icon,icon.unusableAlpha)
					end
				else
					TellMeWhen_SetIconAlpha(icon,icon.unusableAlpha)
				end
			else
				icon:SetAlpha(0)
			end
		end
	end
end

function TellMeWhen_Icon_WpnEnchant_OnEvent(icon, event, ...)
	if ( event == "UNIT_INVENTORY_CHANGED" ) and ( select(1, ...) == "player" ) then
		local slotID;
		if ( icon.WpnEnchantType == "mainhand" ) then
			slotID = GetInventorySlotInfo("MainHandSlot");
		elseif ( icon.WpnEnchantType == "offhand" ) then
			slotID = GetInventorySlotInfo("SecondaryHandSlot");
		end
		local wpnTexture = GetInventoryItemTexture("player", slotID);
		if ( wpnTexture ) then
			icon.texture:SetTexture(wpnTexture);
		else
			icon.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
		end
		icon.startTime = GetTime();
	end
end

function TellMeWhen_Icon_WpnEnchant_OnUpdate(icon, elapsed)
	icon.updateTimer = icon.updateTimer - elapsed;
	if ( icon.updateTimer <= 0 ) then
		icon.updateTimer = TellMeWhen_Settings["Interval"];
		local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo();
		if ( icon.WpnEnchantType == "mainhand" ) and ( hasMainHandEnchant ) then
			TellMeWhen_SetIconAlpha(icon, icon.presentAlpha);
			if ( mainHandCharges > 1 ) then
				icon.countText:SetText(mainHandCharges);
				icon.countText:Show();
			else
				icon.countText:Hide();
			end
			if (icon.ShowTimer) then
				if ( icon.startTime ~= nil ) then
					CooldownFrame_SetTimer(icon.Cooldown, GetTime(), mainHandExpiration/1000, 1);
				else
					icon.startTime = GetTime();
				end
			end
		elseif ( icon.WpnEnchantType == "offhand" ) and ( hasOffHandEnchant ) then
			TellMeWhen_SetIconAlpha(icon, icon.presentAlpha);
			if ( offHandCharges > 1 ) then
				icon.countText:SetText(offHandCharges);
				icon.countText:Show();
			else
				icon.countText:Hide();
			end
			if (icon.ShowTimer) then
				if ( icon.startTime ~= nil ) then
					CooldownFrame_SetTimer(icon.Cooldown, GetTime(), offHandExpiration/1000, 1);
				else
					icon.startTime = GetTime();
				end
			end
		else
			TellMeWhen_SetIconAlpha(icon, icon.absentAlpha);
			CooldownFrame_SetTimer(icon.Cooldown, 0, 0, 0);
		end
	end
end

function TellMeWhen_Icon_Totem_OnEvent(icon, event, ...)
	--Totems have changed! Do something!
	local spellName, spellRank, spellIconPath,totemNames
	totemNames = icon.namelist;
	local foundTotem = false
	for iSlot=1, 4 do
		local haveTotem, totemName, startTime, totemDuration, totemIcon = GetTotemInfo(iSlot)
		for i, iName in ipairs(totemNames) do
			spellName, spellRank, spellIconPath = GetSpellInfo(iName)
			if ( totemName and totemName:find(iName) ) then
				foundTotem = true;
				icon.texture:SetVertexColor(1,1,1,1);
				TellMeWhen_SetIconAlpha(icon, icon.presentAlpha);

				if ( icon.texture:GetTexture() ~= totemIcon ) then
					icon.texture:SetTexture( totemIcon );
					icon.learnedTexture = true;
				--else
					--icon.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
				end

				if ( icon.ShowTimer ) then
					-- The startTime reported here is both cast to an int and off by
					-- a latency meaning it can be significantly low.  So we cache the GetTime
					-- that the totem actually appeared, so long as GetTime is reasonably close to
					-- startTime (since the totems may have been out for awhile before this runs.)
					local precise = GetTime()
					if ( precise - startTime > 1 ) then
						precise = startTime + 1
					end
					CooldownFrame_SetTimer(icon.Cooldown, precise, totemDuration, 1);
				end
				icon:SetScript("OnUpdate", nil);
				break
			end
		end
	end
	if (not foundTotem) then
		if ( icon.absentAlpha == 1 and icon.presentAlpha == 1) then
			icon.texture:SetVertexColor(1,0.35,0.35,1);
		end
		TellMeWhen_SetIconAlpha(icon, icon.absentAlpha);
		CooldownFrame_SetTimer(icon.Cooldown, 0, 0, 0);
	end
end

function TellMeWhen_Icon_ConditionCheck(icon)
	local retCode = true;
	if ( icon.conditionPresent ) then
		for i=1,#icon.conditions do
			local tempRet = TellMeWhen_Icon_ConditionCheckLevel(icon.conditions[i]);
			if( icon.conditions[i].ConditionAndOr == "OR" ) then
				retCode = retCode or tempRet;
			else
				retCode = retCode and tempRet;
			end
		end
	end
	return retCode;
end

function TellMeWhen_Icon_ConditionCheckLevel(condition)
	local retCode = true;
	
	if ( condition.ConditionType == "HEALTH" ) then
		local percent = 100 * UnitHealth("player")/UnitHealthMax("player");
		retCode = TellMeWhen_Icon_ConditionCheckInternal(condition.ConditionOperator, condition.ConditionLevel, percent);
	elseif ( condition.ConditionType == "MANA" ) then
		local percent = 100 * UnitPower("player",0)/UnitPowerMax("player",0);
		retCode = TellMeWhen_Icon_ConditionCheckInternal(condition.ConditionOperator, condition.ConditionLevel, percent);
	elseif ( condition.ConditionType == "RAGE" ) then
		local percent = 100 * UnitPower("player",1)/UnitPowerMax("player",1);
		retCode = TellMeWhen_Icon_ConditionCheckInternal(condition.ConditionOperator, condition.ConditionLevel, percent);
	elseif ( condition.ConditionType == "ENERGY" ) then
		local percent = 100 * UnitPower("player",3)/UnitPowerMax("player",3);
		retCode = TellMeWhen_Icon_ConditionCheckInternal(condition.ConditionOperator, condition.ConditionLevel, percent);
	elseif ( condition.ConditionType == "FOCUS" ) then
		local percent = 100 * UnitPower("player",2)/UnitPowerMax("player",2);
		retCode = TellMeWhen_Icon_ConditionCheckInternal(condition.ConditionOperator, condition.ConditionLevel, percent);
	elseif ( condition.ConditionType == "RUNIC_POWER" ) then
		local percent = 100 * UnitPower("player",6)/UnitPowerMax("player",6);
		retCode = TellMeWhen_Icon_ConditionCheckInternal(condition.ConditionOperator, condition.ConditionLevel, percent);
	end
	
	return retCode;
end

function TellMeWhen_Icon_ConditionCheckInternal(operator, inLevel, percent)
	local retCode = true;
	local level = tonumber(inLevel)
	
	if ( operator == "==" ) then
		retCode = ( percent == level );
	elseif ( operator == "<" ) then
		retCode = ( percent < level );
	elseif ( operator == "<=" ) then
		retCode = ( percent <= level );
	elseif ( operator == ">" ) then
		retCode = ( percent > level );
	elseif ( operator == ">=" ) then
		retCode = ( percent >= level );
	elseif ( operator == "~=" ) then
		retCode = ( percent ~= level );
	end
	return retCode;
end

function TellMeWhen_GetSpellNames(buffName,firstOnly,toname)
	local buffNames = {}
	if (TellMeWhen_BuffEquivalencies[buffName]) then
		 buffNames = TellMeWhen_SplitNames(TellMeWhen_BuffEquivalencies[buffName],"spell");
	else
		 buffNames = TellMeWhen_SplitNames(buffName,"spell")
	end
	if toname then
		return GetSpellInfo(buffNames[1])
	end
	if ( firstOnly ) then
		return buffNames[1]
	end
	return buffNames
end

function TellMeWhen_GetItemIDs(buffName,firstOnly)
	local buffNames = {}
	buffNames = TellMeWhen_SplitNames(buffName,"item")
	if ( firstOnly ) then
		return buffNames[1]
	end
	return buffNames
end

function TellMeWhen_SplitNames(buffName,convertIDs)
	local buffNames = {}
	-- If buffName contains one or more semicolons, split the list into parts
	if (buffName:find(";") ~= nil) then
		buffNames = { strsplit(";", buffName) }
	else
		buffNames = { buffName }
	end
	if (convertIDs == "item") then
		for k,v in ipairs(buffNames) do
			if (tonumber( v ) == nil) then
				local _,itemLink = GetItemInfo(v)
				local itemID
				if itemLink then
					_, _, itemID = string.find(itemLink, ":(%d+)")
				end
				buffNames[k] = itemID
			end
		end
	end
	return buffNames
end

function TellMeWhen_SetIconAlpha(icon, alphaScale)
	icon:SetAlpha( alphaScale * ( icon.baseAlpha / 100.0 ) );
end

function TellMeWhen_SkinCallback(arg, SkinID, Gloss, Backdrop, Group, Button, Colors)
-- when ButtonFacade skin is changed

-- Put info into TellMeWhen_Settings for later
--[[ if not Group then

 else
 	TellMeWhen_Settings[Group]["SkinID"] = SkinID
 	TellMeWhen_Settings[Group]["Gloss"] = Gloss
 	TellMeWhen_Settings[Group]["Backdrop"] = Backdrop
 	TellMeWhen_Settings[Group]["Colors"] = Colors
 end]]


end
